/*************************************************************************************************
 * The command line interface of web crawler
 *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
 * This file is part of Hyper Estraier.
 * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License or any later version.  Hyper Estraier is distributed in the hope
 * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 * License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with Hyper
 * Estraier; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA.
 *************************************************************************************************/


#include "wavermod.h"
#include "myimage.dat"


#define NUMBUFSIZ      32                /* size of a buffer for a number */
#define URIBUFSIZ      8192              /* size of a buffer for an URI */


/* global variables */
const char *g_progname;                  /* program name */
int g_sigterm = FALSE;                   /* flag for termination signal */
const char *g_pxhost = NULL;             /* host name of the proxy */
int g_pxport = 0;                        /* host name of the proxy */
int g_timeout = -1;                      /* timeout in seconds */
int g_inputlang = ESTLANGEN;             /* prefered language */


/* function prototypes */
int main(int argc, char **argv);
static void usage(void);
static void printferror(const char *format, ...);
static void printfinfo(const char *format, ...);
static int runfetch(int argc, char **argv);
static int rununittest(int argc, char **argv);
static int procfetch(const char *url);
static int procunittest(const char *rootdir);
static int strtolang(const char *str);
static char *capitalize(const char *str);


/* main routine */
int main(int argc, char **argv){
  const char *tmp;
  int rv;
  if((tmp = getenv("ESTDBGFD")) != NULL) dpdbgfd = atoi(tmp);
  cbstdiobin();
  g_progname = argv[0];
  g_sigterm = FALSE;
  if(!est_init_net_env()){
    printferror("could not initialize network environment");
    exit(1);
  }
  atexit(est_free_net_env);
  if(argc < 2) usage();
  rv = 0;
  if(!strcmp(argv[1], "fetch")){
    rv = runfetch(argc, argv);
  } else if(!strcmp(argv[1], "unittest")){
    rv = rununittest(argc, argv);
  } else {
    usage();
  }
  return rv;
}


/* print the usage and exit */
static void usage(void){
  fprintf(stderr, "%s: command line interface of web crawler\n", g_progname);
  fprintf(stderr, "\n");
  fprintf(stderr, "usage:\n");
  fprintf(stderr, "  %s fetch [-proxy host port] [-tout num] [-il lang] url\n", g_progname);
  fprintf(stderr, "  %s unittest rootdir\n", g_progname);
  fprintf(stderr, "\n");
  exit(1);
}


/* print formatted error string and flush the buffer */
static void printferror(const char *format, ...){
  va_list ap;
  va_start(ap, format);
  fprintf(stderr, "%s: ERROR: ", g_progname);
  vfprintf(stderr, format, ap);
  fputc('\n', stderr);
  fflush(stderr);
  va_end(ap);
}


/* print formatted information string and flush the buffer */
static void printfinfo(const char *format, ...){
  va_list ap;
  va_start(ap, format);
  printf("%s: INFO: ", g_progname);
  vprintf(format, ap);
  putchar('\n');
  fflush(stdout);
  va_end(ap);
}


/* parse arguments of the fetch command */
static int runfetch(int argc, char **argv){
  char *url;
  int i, rv;
  url = NULL;
  for(i = 2; i < argc; i++){
    if(!url && argv[i][0] == '-'){
      if(!strcmp(argv[i], "-proxy")){
        if(++i >= argc) usage();
        g_pxhost = argv[i];
        if(++i >= argc) usage();
        g_pxport = atoi(argv[i]);
      } else if(!strcmp(argv[i], "-tout")){
        if(++i >= argc) usage();
        g_timeout = atoi(argv[i]);
      } else if(!strcmp(argv[i], "-il")){
        if(++i >= argc) usage();
        g_inputlang = strtolang(argv[i]);
      } else {
        usage();
      }
    } else if(!url){
      url = argv[i];
    } else {
      usage();
    }
  }
  if(!url) usage();
  rv = procfetch(url);
  return rv;
}


/* parse arguments of the unittest command */
static int rununittest(int argc, char **argv){
  char *rootdir;
  int i, rv;
  rootdir = NULL;
  for(i = 2; i < argc; i++){
    if(!rootdir && argv[i][0] == '-'){
      usage();
    } else if(!rootdir){
      rootdir = argv[i];
    } else {
      usage();
    }
  }
  if(!rootdir) usage();
  rv = procunittest(rootdir);
  return rv;
}


/* perform the fetch command */
static int procfetch(const char *url){
  CBMAP *heads;
  CBLIST *links;
  CBDATUM *raw;
  ESTDOC *doc;
  const char *border, *vbuf;
  char *str;
  int i, vsiz;
  raw = cbdatumopen("", 0);
  heads = cbmapopen();
  links = cblistopen();
  doc = est_doc_new();
  if(!fetch_document(url, g_pxhost, g_pxport, g_timeout, raw, heads, links, doc, g_inputlang)){
    est_doc_delete(doc);
    cblistclose(links);
    cbmapclose(heads);
    cbdatumclose(raw);
    return 1;
  }
  border = est_border_str();
  printf("URL: %s\r\n", url);
  str = cbdatestrhttp(time(NULL), 0);
  printf("Date: %s\r\n", str);
  free(str);
  str = cbmimeencode((vbuf = est_doc_attr(doc, ESTDATTRTITLE)) ? vbuf : url, "UTF-8", TRUE);
  printf("Subject: [estwaver] %s\r\n", str);
  free(str);
  printf("Content-Type: multipart/mixed; boundary=%s\r\n", border);
  printf("\r\n");
  printf("This is a multi-part message in MIME format.\n");
  printf("\r\n");
  printf("--%s\r\n", border);
  printf("Content-Type: text/x-estraier-draft\r\n");
  printf("X-Estwaver-Role: draft\r\n");
  printf("\r\n");
  str = est_doc_dump_draft(doc);
  printf("%s", str);
  free(str);
  printf("\r\n");
  printf("--%s\r\n", border);
  printf("Content-Type: text/plain\r\n");
  printf("X-Estwaver-Role: links\r\n");
  printf("\r\n");
  for(i = 0; i < cblistnum(links); i++){
    printf("%s\n", cblistval(links, i, NULL));
  }
  printf("\r\n");
  printf("--%s\r\n", border);
  cbmapiterinit(heads);
  while((vbuf = cbmapiternext(heads, &vsiz)) != NULL){
    if(vsiz < 1){
      printf("X-Original-HTTP-Response: %s\r\n", cbmapget(heads, vbuf, vsiz, NULL));
    } else if(!strcmp(vbuf, "content-encoding")){
      printf("X-Original-Content-Encoding: %s\r\n", cbmapget(heads, vbuf, vsiz, NULL));
    } else {
      str = capitalize(vbuf);
      printf("%s: %s\r\n", str, cbmapget(heads, vbuf, vsiz, NULL));
      free(str);
    }
  }
  printf("X-Estwaver-Role: raw\r\n");
  printf("\r\n");
  fwrite(cbdatumptr(raw), 1, cbdatumsize(raw), stdout);
  printf("\r\n");
  printf("--%s--\r\n", border);
  est_doc_delete(doc);
  cblistclose(links);
  cbmapclose(heads);
  cbdatumclose(raw);
  return 0;
}


/* perform the unittest command */
static int procunittest(const char *rootdir){
  QUEUE *queue;
  CBMAP *kwmap;
  char uri[URIBUFSIZ], *vbuf;
  int i, err;
  unlink(rootdir);
  if(!(queue = queue_open(rootdir))){
    printferror("%s: could not open", rootdir);
    return 1;
  }
  err = FALSE;
  printfinfo("checking priority queue");
  for(i = 0; i < 100; i++){
    sprintf(uri, "http://%05d/%x/%x.html",
            i + 1, (int)(est_random() * 0x1000000), (int)(est_random() * 0x1000000));
    if(!queue_enqueue(queue, uri, est_random())){
      err = TRUE;
      break;
    }
    if(i % 10 == 0) queue_set_range(queue, i);
  }
  if(err){
    printferror("%s: enqueue failed", rootdir);
    queue_close(queue);
    return 1;
  }
  for(i = 0; (vbuf = queue_dequeue(queue)) != NULL; i++){
    free(vbuf);
  }
  if(i != 100){
    printferror("%s: dequeue failed", rootdir);
    queue_close(queue);
    return 1;
  }
  if(!queue_close(queue)){
    printferror("%s: could not close", rootdir);
    return 1;
  }
  printfinfo("checking keyword map");
  kwmap = cbmapopen();
  for(i = 0; i < 10000; i++){
    sprintf(uri, "%d", (int)(est_random() * 1000));
    kwmap_add(kwmap, uri, (int)(est_random() * 1000));
  }
  kwmap_reduce(kwmap, 100);

  printf("%d\n", cbmaprnum(kwmap));


  cbmapclose(kwmap);

  return 0;
}


/* get the language value */
static int strtolang(const char *str){
  if(!cbstricmp(str, "en")) return ESTLANGEN;
  if(!cbstricmp(str, "ja")) return ESTLANGJA;
  if(!cbstricmp(str, "zh")) return ESTLANGZH;
  if(!cbstricmp(str, "ko")) return ESTLANGKO;
  return ESTLANGMISC;
}


/* make a capitalized string */
static char *capitalize(const char *str){
  char *buf;
  int i, cap;
  buf = cbmemdup(str, -1);
  cap = TRUE;
  for(i = 0; buf[i] != '\0'; i++){
    if(cap && buf[i] >= 'a' && buf[i] <= 'z'){
      buf[i] -= 'a' - 'A';
    }
    cap = buf[i] == ' ' || buf[i] == '-';
  }
  return buf;
}



/* END OF FILE */
